import type { FunctionComponent } from "react";
import { Color3, Color4 } from "@babylonjs/core/Maths/math.color.js";
import type { BaseComponentProps } from "../hoc/propertyLine.js";
export type ColorPickerProps<C extends Color3 | Color4> = {
    isLinearMode?: boolean;
} & BaseComponentProps<C>;
export declare const ColorPickerPopup: FunctionComponent<ColorPickerProps<Color3 | Color4>>;
type HsvKey = "h" | "s" | "v";
export type InputHexProps = BaseComponentProps<Color3 | Color4> & {
    label?: string;
    linearHex?: boolean;
    isLinearMode?: boolean;
};
/**
 * Component which displays the passed in color's HEX value, either in linearSpace (if linearHex is true) or in gamma space
 * When the hex color is changed by user, component calculates the new Color3/4 value and calls onChange
 *
 * Component uses the isLinearMode boolean to display an informative label regarding linear / gamma space
 * @param props - The properties for the InputHexField component.
 * @returns
 */
export declare const InputHexField: FunctionComponent<InputHexProps>;
type InputHsvFieldProps = {
    color: Color3 | Color4;
    label: string;
    hsvKey: HsvKey;
    onChange: (color: Color3 | Color4) => void;
    max: number;
    scale?: number;
};
/**
 * In the HSV (Hue, Saturation, Value) color model, Hue (H) ranges from 0 to 360 degrees, representing the color's position on the color wheel.
 * Saturation (S) ranges from 0 to 100%, indicating the intensity or purity of the color, with 0 being shades of gray and 100 being a fully saturated color.
 * Value (V) ranges from 0 to 100%, representing the brightness of the color, with 0 being black and 100 being the brightest.
 * @param props - The properties for the InputHsvField component.
 */
export declare const InputHsvField: FunctionComponent<InputHsvFieldProps>;
export {};
