import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
/* eslint-disable jsdoc/require-returns */
/* eslint-disable @typescript-eslint/naming-convention */
import { useState, useEffect, useCallback } from "react";
import { Input, Label, SpinButton, useId, ColorPicker, ColorSlider, ColorArea, AlphaSlider, InfoLabel, Link, makeStyles, Popover, PopoverSurface, PopoverTrigger, tokens, Body1Strong, } from "@fluentui/react-components";
import { Color3, Color4 } from "@babylonjs/core/Maths/math.color.js";
const useColorPickerStyles = makeStyles({
    colorPickerContainer: {
        width: "300px",
        display: "flex",
        flexDirection: "column",
        gap: tokens.spacingVerticalMNudge, // 10px
        overflow: "visible",
    },
    triggerColor: {
        width: "50px",
        height: tokens.lineHeightBase100,
        borderRadius: tokens.borderRadiusMedium,
        cursor: "pointer", // Show pointer on hover
        border: `${tokens.spacingVerticalXXS} solid ${tokens.colorNeutralShadowKeyLighter}`,
        "@media (forced-colors: active)": {
            forcedColorAdjust: "none", // ensures element maintains color in high contrast mode
        },
    },
    previewColor: {
        width: "50px",
        height: "50px",
        borderRadius: tokens.borderRadiusMedium,
        border: `${tokens.spacingVerticalXXS} solid ${tokens.colorNeutralShadowKeyLighter}`,
        "@media (forced-colors: active)": {
            forcedColorAdjust: "none", // ensures elmement maintains color in high constrast mode
        },
    },
    row: {
        display: "flex",
        flexDirection: "row",
        gap: tokens.spacingVerticalM, // 12px
        alignItems: "center",
    },
    colorFieldWrapper: {
        display: "flex",
        flexDirection: "column",
        gap: tokens.spacingVerticalSNudge, // 6px
    },
    input: {
        width: "80px",
    },
    spinButton: {
        minWidth: "60px",
    },
    container: {
        display: "flex",
        flexDirection: "column",
        gap: tokens.spacingVerticalL, // 16px
    },
});
export const ColorPickerPopup = (props) => {
    const classes = useColorPickerStyles();
    const [color, setColor] = useState(props.value);
    const [popoverOpen, setPopoverOpen] = useState(false);
    useEffect(() => {
        props.onChange(color); // Ensures the parent is notified when color changes from within colorPicker
    }, [color]);
    useEffect(() => {
        setColor(props.value); // Ensures the trigger color updates when props.value changes
    }, [props.value]);
    const handleChange = (_, data) => {
        let color = Color3.FromHSV(data.color.h, data.color.s, data.color.v);
        if (props.value instanceof Color4) {
            color = Color4.FromColor3(color, data.color.a ?? 1);
        }
        setColor(color);
    };
    return (_jsxs(Popover, { positioning: {
            align: "start",
            overflowBoundary: document.body,
            autoSize: true,
        }, open: popoverOpen, trapFocus: true, onOpenChange: (_, data) => setPopoverOpen(data.open), children: [_jsx(PopoverTrigger, { disableButtonEnhancement: true, children: _jsx("div", { className: classes.triggerColor, style: { backgroundColor: color.toHexString() } }) }), _jsx(PopoverSurface, { children: _jsxs("div", { className: classes.colorPickerContainer, children: [_jsxs(ColorPicker, { color: rgbaToHsv(color), onColorChange: handleChange, children: [_jsx(ColorArea, { inputX: { "aria-label": "Saturation" }, inputY: { "aria-label": "Brightness" } }), _jsx(ColorSlider, { "aria-label": "Hue" }), color instanceof Color4 && _jsx(AlphaSlider, { "aria-label": "Alpha" })] }), _jsxs("div", { className: classes.container, children: [_jsxs("div", { className: classes.row, children: [_jsx("div", { className: classes.previewColor, style: { backgroundColor: color.toHexString() } }), _jsx(InputHexField, { label: "Gamma Hex", value: color, isLinearMode: props.isLinearMode, onChange: setColor }), _jsx(InputHexField, { label: "Linear Hex", linearHex: true, isLinearMode: props.isLinearMode, value: color, onChange: setColor })] }), _jsxs("div", { className: classes.row, children: [_jsx(InputRgbField, { label: "Red", color: color, rgbKey: "r", onChange: setColor }), _jsx(InputRgbField, { label: "Green", color: color, rgbKey: "g", onChange: setColor }), _jsx(InputRgbField, { label: "Blue", color: color, rgbKey: "b", onChange: setColor }), _jsx(InputAlphaField, { color: color, onChange: setColor })] }), _jsxs("div", { className: classes.row, children: [_jsx(InputHsvField, { label: "Hue", color: color, hsvKey: "h", max: 360, onChange: setColor }), _jsx(InputHsvField, { label: "Saturation", color: color, hsvKey: "s", max: 100, scale: 100, onChange: setColor }), _jsx(InputHsvField, { label: "Value", color: color, hsvKey: "v", max: 100, scale: 100, onChange: setColor })] })] })] }) })] }));
};
/**
 * Component which displays the passed in color's HEX value, either in linearSpace (if linearHex is true) or in gamma space
 * When the hex color is changed by user, component calculates the new Color3/4 value and calls onChange
 *
 * Component uses the isLinearMode boolean to display an informative label regarding linear / gamma space
 * @param props - The properties for the InputHexField component.
 * @returns
 */
export const InputHexField = (props) => {
    const id = useId("hex-input");
    const styles = useColorPickerStyles();
    const { label, value, onChange, linearHex, isLinearMode } = props;
    const handleChange = (e, _) => {
        // If linearHint (aka PBR material, ensure the other values are displayed in gamma even if linear hex changes)
        const value = e.target.value;
        if (value != "" && /^[0-9A-Fa-f]+$/g.test(value) == false) {
            return;
        }
        onChange(Color3.FromHexString(value).toGammaSpace());
    };
    return (_jsxs("div", { className: styles.colorFieldWrapper, children: [props.linearHex ? (_jsx(InfoLabel, { htmlFor: id, info: !isLinearMode ? (_jsx(_Fragment, { children: " This color picker is attached to an entity whose color is stored in gamma space, so we are showing linear hex in disabled view " })) : (_jsxs(_Fragment, { children: ["This color picker is attached to an entity whose color is stored in linear space (ex: PBR Material), and Babylon converts the color to gamma space before rendering on screen because the human eye is best at processing colors in gamma space. We thus also want to display the color picker in gamma space so that the color chosen here will match the color seen in your entity.", _jsx("br", {}), "If you want to copy/paste the HEX into your code, you can either use", _jsx(Body1Strong, { children: "Color3.FromHexString(LINEAR_HEX)" }), _jsx("br", {}), "or", _jsx("br", {}), _jsx(Body1Strong, { children: "Color3.FromHexString(GAMMA_HEX).toLinearSpace()" }), _jsx("br", {}), _jsx("br", {}), _jsx(Link, { href: "https://doc.babylonjs.com/preparingArtForBabylon/controllingColorSpace/", children: " Read more in our docs! " })] })), children: label })) : (_jsx(Label, { htmlFor: id, children: label })), _jsx(Input, { disabled: linearHex ? !isLinearMode : false, className: styles.input, value: linearHex ? value.toLinearSpace().toHexString() : value.toHexString(), id: id, onChange: handleChange })] }));
};
const InputRgbField = (props) => {
    const { color, onChange, label, rgbKey } = props;
    const id = useId(`${label.toLowerCase()}-input`);
    const classes = useColorPickerStyles();
    const handleChange = useCallback((_, data) => {
        const val = data.value ?? parseFloat(data.displayValue ?? "");
        if (val === null || Number.isNaN(val) || !NUMBER_REGEX.test(val.toString())) {
            return;
        }
        const newColor = color.clone();
        newColor[rgbKey] = val / 255.0; // Convert to 0-1 range
        onChange(newColor);
    }, [color]);
    return (_jsxs("div", { className: classes.colorFieldWrapper, children: [_jsx(Label, { htmlFor: id, children: label }), _jsx(SpinButton, { className: classes.spinButton, min: 0, max: 255, value: color[rgbKey] * 255.0, step: 1, id: id, onChange: handleChange, name: rgbKey })] }));
};
/**
 * In the HSV (Hue, Saturation, Value) color model, Hue (H) ranges from 0 to 360 degrees, representing the color's position on the color wheel.
 * Saturation (S) ranges from 0 to 100%, indicating the intensity or purity of the color, with 0 being shades of gray and 100 being a fully saturated color.
 * Value (V) ranges from 0 to 100%, representing the brightness of the color, with 0 being black and 100 being the brightest.
 * @param props - The properties for the InputHsvField component.
 */
export const InputHsvField = (props) => {
    const { color, label, hsvKey, max, scale = 1 } = props;
    const id = useId(`${label.toLowerCase()}-input`);
    const classes = useColorPickerStyles();
    const handleChange = useCallback((_, data) => {
        const val = data.value ?? parseFloat(data.displayValue ?? "");
        if (val === null || Number.isNaN(val) || !NUMBER_REGEX.test(val.toString())) {
            return;
        }
        // Convert current color to HSV, update the new hsv value, then call onChange prop
        const hsv = rgbaToHsv(color);
        hsv[hsvKey] = val / scale;
        let newColor = Color3.FromHSV(hsv.h, hsv.s, hsv.v);
        if (color instanceof Color4) {
            newColor = Color4.FromColor3(newColor, color.a ?? 1);
        }
        props.onChange(newColor);
    }, [color]);
    return (_jsxs("div", { className: classes.colorFieldWrapper, children: [_jsx(Label, { htmlFor: id, children: label }), _jsx(SpinButton, { className: classes.spinButton, min: 0, max: max, value: rgbaToHsv(color)[hsvKey] * scale, step: 1, id: id, onChange: handleChange, name: hsvKey })] }));
};
/**
 * Displays the alpha value of a color, either in the disabled state (if color is Color3) or as a spin button (if color is Color4).
 * @param props
 * @returns
 */
const InputAlphaField = (props) => {
    const classes = useColorPickerStyles();
    const id = useId("alpha-input");
    const { color } = props;
    const onChange = (_, data) => {
        const value = data.value ?? parseFloat(data.displayValue ?? "");
        if (Number.isNaN(value) || value < 0 || value > 1) {
            return;
        }
        if (color instanceof Color4) {
            const newColor = color.clone();
            newColor.a = value;
            return newColor;
        }
        else {
            return Color4.FromColor3(color, value);
        }
    };
    return (_jsxs("div", { className: classes.colorFieldWrapper, children: [_jsxs("div", { className: classes.row, children: [_jsx(Label, { htmlFor: id, children: "Alpha" }), color instanceof Color3 && (_jsx(InfoLabel, { htmlFor: id, info: _jsx(_Fragment, { children: "Because this color picker is representing a Color3, we do not permit modifying alpha from the color picker. You can however modify the material's alpha property directly, either in code via material.alpha OR via inspector's transparency section." }) }))] }), _jsx(SpinButton, { disabled: color instanceof Color3, min: 0, max: 1, className: classes.spinButton, value: color instanceof Color3 ? 1 : color.a, step: 0.01, onChange: onChange, id: id })] }));
};
const NUMBER_REGEX = /^\d+$/;
function rgbaToHsv(color) {
    const c = new Color3(color.r, color.g, color.b);
    const hsv = c.toHSV();
    return { h: hsv.r, s: hsv.g, v: hsv.b, a: color.a };
}
//# sourceMappingURL=colorPicker.js.map